import pandas as pd
import sqlalchemy as sa
from sqlalchemy.orm import sessionmaker
import numpy as np
import config


class FactorDetailDoc(object):
    def __init__(self):
        db_url = '''mysql+mysqlconnector://{0}:{1}@{2}:{3}/{4}'''.format(config.rl_db_user,
                                                                         config.rl_db_pwd,
                                                                         config.rl_db_host,
                                                                         config.rl_db_port,
                                                                         config.rl_db_database)
        self._destination = sa.create_engine(db_url)
        self._destsession = sessionmaker(bind=self._destination, autocommit=False, autoflush=True)
        self._factor_detail_columns = ['factor_type', 'factor_name', 'factor_en_name', 'factor_cn_name', 'direction',
                                       'description', 'latxt_description', 'unit', 'view_dimension', 'tag']

    def _exec_sql(self, sql):
        session = self._destsession()
        session.execute(sql)
        session.commit()
        session.close()

    def update_factor_info(self, df):
        sql = 'update factor_info set flag = null,is_verify=2'
        self._exec_sql(sql)
        df.to_sql(name='factor_info', con=self._destination, if_exists='append', index=False)

    def update_factor_detail(self, df):
        df = df[self._factor_detail_columns]
        df['tag'] = df['tag'].fillna(0)
        df['tag'] = df['tag'].astype('int').astype('str')
        df['tag'] = df['tag'].replace('0', np.nan)
        df.to_sql(name='factor_detail', con=self._destination, if_exists='append', index=False)

    def update(self):
        excel = pd.read_excel('doc/factor_detail_doc.xlsx', sheet_name=None)
        sheet_names = list(excel.keys())
        factor_info = sheet_names.pop(0)
        self.update_factor_info(excel[factor_info])
        sql = f'update factor_detail set flag = null,is_verify=2'
        self._exec_sql(sql)
        for sheet_name in sheet_names:
            self.update_factor_detail(excel[sheet_name])
        print('complete!')


if __name__ == '__main__':
    fd = FactorDetailDoc()
    fd.update()
